package com.persagy.dmp.file.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.dmp.common.constant.ResponseCode;
import com.persagy.dmp.common.exception.BusinessException;
import com.persagy.dmp.common.model.response.CommonResult;
import com.persagy.dmp.common.utils.ResultHelper;
import com.persagy.dmp.file.dao.FileMapper;
import com.persagy.dmp.file.domain.FileRequestData;
import com.persagy.dmp.file.model.FileInfo;
import com.persagy.dmp.file.model.FileInfoCreator;
import com.persagy.dmp.file.service.FileUploadService;
import com.persagy.dmp.storage.service.MinioCommonService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/***
 * Description: 文件上传逻辑处理类
 * @author : lijie
 * @date :2021/10/9 20:09
 * Update By lijie 2021/10/9 20:09
 */
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@Slf4j
public class FileUploadServiceImpl extends ServiceImpl<FileMapper, FileInfo> implements FileUploadService {

   private final MinioCommonService minioCommonService;

    /**上传成功*/
    private static final Integer UPLOAD_SUCCESS = 0;

    /**部分上传*/
    private static final Integer UPLOAD_PART = 1;

    /***
     * Description: 初始化文件上传路径列表
     * @param requestData : 请求参数
     * @return : java.util.List<com.persagy.dmp.file.model.FileInfo>
     * @author : lijie
     * @date :2021/10/9 20:27
     * Update By lijie 2021/10/9 20:27
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<FileInfo> initChunkUpload(FileRequestData requestData) {
        if (StrUtil.isBlank(requestData.getFileMd5())){
            throw new BusinessException(ResponseCode.A0400.getCode(), ResponseCode.A0400.getDesc());
        }
        // 1.根据文件md5值查询数据库
        FileInfo fileInfo = queryFileInfosByFileMd5(requestData.getFileMd5());
        // 2.历史已上传且已完成上传
        if (ObjectUtil.isNotNull(fileInfo)
                && UPLOAD_SUCCESS.equals(fileInfo.getUploadStatus())){
            return CollUtil.newArrayList(fileInfo);
        }
        // 3.历史已上传
        //获取到该文件已上传分片
        Map<Integer,String> okChunkMap = minioCommonService.mapChunkObjectNames(requestData.getBucketName(),
                requestData.getFileMd5());
        List<FileInfo> chunkUploadUrls = new ArrayList<>();
        if (okChunkMap.size() > 0){
            for (int i = 1; i <= requestData.getChunkCount(); i++) {
                //判断当前分片是否已经上传过了
                if(okChunkMap.containsKey(i)){
                    continue;
                }
                //生成分片上传url
                FileInfo url = new FileInfo();
                url.setPartNumber(i);
                url.setUploadUrl(minioCommonService.createUploadChunkUrl(requestData.getBucketName(),
                        requestData.getFileMd5(),i));
                chunkUploadUrls.add(url);
            }
            if (chunkUploadUrls.size() == 0){
                return new ArrayList<>();
            }
            return chunkUploadUrls;
        }
        // 4.第一次上传或已存在但没有上传过分片的
        return firstInitChunkUploadInternal(requestData);
    }
    /***
     * Description: 初次上传和已有文件信息但未上传任何分片的情况下则直接生成所有上传url 
     * @param requestData : 
     * @return : java.util.List<com.persagy.dmp.file.model.FileInfo>  
     * @author : lijie
     * @date :2021/10/9 20:36
     * Update By lijie 2021/10/9 20:36
     */
    private List<FileInfo> firstInitChunkUploadInternal(FileRequestData requestData) {
        List<String> uploadUrls = minioCommonService.createUploadChunkUrlList(requestData.getBucketName(),
                requestData.getFileMd5(),
                requestData.getChunkCount());
        List<FileInfo> chunkUploadUrls = new ArrayList<>();
        for (int i = 1; i <= uploadUrls.size(); i++) {
            FileInfo fileInfo = new FileInfo();
            fileInfo.setPartNumber(i);
            fileInfo.setUploadUrl(minioCommonService.createUploadChunkUrl(requestData.getBucketName(),
                    requestData.getFileMd5(),i));
            chunkUploadUrls.add(fileInfo);
        }
        //向数据库中记录该文件的上传信息
        FileInfo fileInfo = FileInfoCreator.of(requestData.getGroupCode(),requestData.getBusinessId(),
                requestData.getBucketName(), requestData.getFileName());
        fileInfo.setUploadStatus(UPLOAD_PART);
        fileInfo.setFileMd5(requestData.getFileMd5());
        this.save(fileInfo);
        return chunkUploadUrls;
    }

    /***
     * Description: 根据文件md5查询文件信息
     * @param fileMd5 : 文件md5值
     * @return : java.util.List<com.persagy.dmp.file.model.FileInfo>
     * @author : lijie
     * @date :2021/10/9 20:32
     * Update By lijie 2021/10/9 20:32
     */
    private FileInfo queryFileInfosByFileMd5(String fileMd5) {
        LambdaQueryWrapper<FileInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(FileInfo::getFileMd5,fileMd5);
        queryWrapper.eq(FileInfo::getValid,true);
        return this.getOne(queryWrapper);
    }
}
